/*                                                                                                            
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://eu-egee.org/partners/ for details on the copyright holders.
 * For license conditions see the license file or
 * http://eu-egee.org/license.html
 */

/*
 * Copyright (c) 2001 EU DataGrid.
 * For license conditions see http://www.eu-datagrid.org/license.html
 *
 * Copyright (c) 2001, 2002, 2003 by
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 */  

/*****************************************************************************
                            Include header files
******************************************************************************/

#include "jobrep_config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sql.h>
#include <sqlext.h>
#include <sqltypes.h>
#include "jobrep_api.h"

 
/******************************************************************************
                                Definitions
******************************************************************************/
 
/******************************************************************************
                          Module specific prototypes
******************************************************************************/
 
/******************************************************************************
                       Define module specific variables
******************************************************************************/  

SQLHENV                  jobrep_henv;  //   <FONT COLOR="green">// Handle ODBC environment</FONT>
long                     V_OD_erg;  //   <FONT COLOR="green">// result of functions</FONT>
SQLHDBC                  jobrep_hdbc; //   <FONT COLOR="green">// Handle connection</FONT>

char                     V_OD_stat[10];// <FONT COLOR="green">// Status SQL</FONT>
SQLINTEGER               V_OD_err,V_OD_rowanz,V_OD_id;
SQLSMALLINT              V_OD_mlen;
char                     V_OD_msg[200],V_OD_buffer[200];


SQLHENV jobrep_henv;
SQLHDBC jobrep_hdbc;
SQLHSTMT jobrep_hstmt;
int jobrep_connected;

TParamType * globalParameters = NULL;



/* 
    ODBC Test function
    This function will test the ability of connecting to a database
*/

int ODBC_Test (char * dbname, char * username, char * password)
{
    int connResult = 0;


    connResult = ODBC_Connect (dbname, username, password);


    if (jobrep_connected)
    {
        ODBC_Disconnect;
    }

    return connResult;
}


/*
 *  Connect to the datasource
 *
 *  The connect string can have the following parts and they refer to
 *  the values in the odbc.ini file
 *
 *	DSN=<data source name>		[mandatory]
 *	HOST=<server host name>		[optional - value of Host]
 *	SVT=<database server type>	[optional - value of ServerType]
 *	DATABASE=<database path>	[optional - value of Database]
 *	OPTIONS=<db specific opts>	[optional - value of Options]
 *	UID=<user name>			[optional - value of LastUser]
 *	PWD=<password>			[optional]
 *	READONLY=<N|Y>			[optional - value of ReadOnly]
 *	FBS=<fetch buffer size>		[optional - value of FetchBufferSize]
 *
 *   Examples:
 *
 *	HOST=star;SVT=Informix 5;UID=demo;PWD=demo;DATABASE=stores5
 *
 *	DSN=stores5_informix;PWD=demo
 */



int ODBC_Connect (char * dbname, char * username, char * password)
{
        V_OD_erg=SQLAllocHandle(SQL_HANDLE_ENV,SQL_NULL_HANDLE,&jobrep_henv);
        if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO))
        {
                ODBC_Errors ("Error AllocHandle");
                return -1;
        }
        V_OD_erg=SQLSetEnvAttr(jobrep_henv, SQL_ATTR_ODBC_VERSION,
                               (void*)SQL_OV_ODBC3, 0);
        if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO))
        {
                ODBC_Errors ("Error SetEnv");
                SQLFreeHandle(SQL_HANDLE_ENV, jobrep_henv);
                return -2;
        }
        V_OD_erg = SQLAllocHandle(SQL_HANDLE_DBC, jobrep_henv, &jobrep_hdbc);
        if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO))
        {
                ODBC_Errors ("Error AllocHDB");
                SQLFreeHandle(SQL_HANDLE_ENV, jobrep_henv);
                return -3;
        }
        SQLSetConnectAttr(jobrep_hdbc, SQL_LOGIN_TIMEOUT, (SQLPOINTER *)5, 0);
        V_OD_erg = SQLConnect(jobrep_hdbc, (SQLCHAR*) dbname, SQL_NTS,
                                     (SQLCHAR*) username, SQL_NTS,
                                     (SQLCHAR*) password, SQL_NTS);
        if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO))
        {
                ODBC_Errors ("Error SQLConnect");
                SQLGetDiagRec(SQL_HANDLE_DBC, jobrep_hdbc,1,
                              V_OD_stat, &V_OD_err,V_OD_msg,100,&V_OD_mlen);
                ODBC_Errors (V_OD_msg);
                SQLFreeHandle(SQL_HANDLE_ENV, jobrep_henv);
                return -4;
        }

        // Connected!
        return 0;
}


/*
int ODBC_Connect (char *connStr)
{
    short buflen;
    char buf[257];
    SQLCHAR dataSource[120];
    //SQLCHAR dsn[33];
    //SQLCHAR desc[255];
    SQLCHAR driverInfo[255];
    //SWORD len1, len2;
    SWORD len1;
    int status;

    if (SQLAllocHandle (SQL_HANDLE_ENV, NULL, &jobrep_henv) != SQL_SUCCESS)
        return -1;

    

    SQLSetEnvAttr (jobrep_henv, SQL_ATTR_ODBC_VERSION, (SQLPOINTER) SQL_OV_ODBC3, SQL_IS_UINTEGER);

    if (SQLAllocHandle (SQL_HANDLE_DBC, jobrep_henv, &jobrep_hdbc) != SQL_SUCCESS)
       return -2;

    // Show the version number of the driver manager 
    status = SQLGetInfo (jobrep_hdbc, SQL_DM_VER, driverInfo, sizeof (driverInfo), &len1);
    if (status != SQL_SUCCESS)
    {
        fprintf (stderr, "Driver Manager: %s\n", driverInfo);
        return -3;
    }


    if (connStr && *connStr)
        strcpy ((char *) dataSource, connStr);

    status = SQLDriverConnect (jobrep_hdbc, 0, (UCHAR *) dataSource, SQL_NTS, (UCHAR *) buf, sizeof (buf), &buflen, SQL_DRIVER_COMPLETE);

    if (status != SQL_SUCCESS && status != SQL_SUCCESS_WITH_INFO)
        return -4;

    // Turn AUTOCOMMIT off so SQL statements can be grouped into a
    // transaction
    status = SQLSetConnectOption(jobrep_hdbc, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_OFF);
    if (status != SQL_SUCCESS) 
    {
        fprintf(stderr, "Unable to turn AUTOCOMMIT off (status=%d)\n", status);
        return -5;
    }

    //SQLSetConnectOption (hdbc, SQL_TXN_ISOLATION_OPTION, SQL_TRANSACTION_SERIALIZABLE);
    //SQLSetConnectOption (hdbc, SQL_TXN_ISOLATION_OPTION, SQL_TRANSACTION_REPEATABLE_READ);
    SQLSetConnectOption (jobrep_hdbc, SQL_TXN_ISOLATION_OPTION, SQL_TRANSACTION_READ_UNCOMMITTED);
    //SQLSetConnectOption (hdbc, SQL_TXN_ISOLATION_OPTION, SQL_TXN_READ_COMMITTED);

    SQLSetConnectOption (jobrep_hdbc, SQL_OPT_TRACEFILE, (UDWORD) "\\SQL.LOG");
    jobrep_connected = 1;

    if (SQLAllocHandle (SQL_HANDLE_STMT, jobrep_hdbc, &jobrep_hstmt) != SQL_SUCCESS)
        return -6;

//    SQLSetStmtOption(hstmt, SQL_CONCURRENCY, SQL_CONCUR_VALUES);

    return 0;
}

*/


/******************************************************************************
Function:   ODBC_Disconnect
Description:
    Disconnect from the database 
Parameters:
Returns:
    0         : succes
    1 or else : failure
******************************************************************************/

int ODBC_Disconnect (void)
{
    if (jobrep_hstmt)
    {
        int sts;
        sts = SQLCloseCursor (jobrep_hstmt);
        if (sts != SQL_ERROR)
	    ODBC_Errors ("CloseCursor");
        SQLFreeHandle (SQL_HANDLE_STMT, jobrep_hstmt);
    }

    if (jobrep_connected)
        SQLDisconnect (jobrep_hdbc);

    if (jobrep_hdbc)
        SQLFreeHandle (SQL_HANDLE_DBC, jobrep_hdbc);

    if (jobrep_henv)
        SQLFreeHandle (SQL_HANDLE_ENV, jobrep_henv);

    return 0;
}


/******************************************************************************
Function:   ODBC_MesgHandler
Description:
    This is the message handler for the communications layer.

    The messages received here are not passed through SQLError,
    because they might occur when no connection is established.
  
    Typically, Rejections from oplrqb are trapped here, and
    also RPC errors.
  
    When no message handler is installed, the messages are output to stderr

Parameters:
    char *reason : The Message or Reason for this to happen
Returns:
    None
******************************************************************************/

void ODBC_MesgHandler (char *reason)
{
    fprintf (stderr, "ODBC_MesgHandler: %s\n", reason);
}


/******************************************************************************
Function:   ODBC_Errors
Description:
    Show all the error information that is available
 
Parameters:
    char *where : Location of failure
Returns:
    0          : success
    -1 or else : failure
******************************************************************************/

char * ODBC_Errors (char *where)
{
    unsigned char   buf[250];
    unsigned char   sqlstate[15];
    unsigned char * search;
              int   i = 0;
    static   char * errorbuf = NULL;

    /*
     *  Get statement errors
     */
    while (SQLError (jobrep_henv, jobrep_hdbc, jobrep_hstmt, sqlstate, NULL, buf, sizeof(buf), NULL) == SQL_SUCCESS)
    {
        search = strdup(buf);
        for (i = 0; i < 250; i++)
        {
            if (strncmp(search, "Duplicate", strlen("Duplicate")) == 0)
                return NULL;

            search++;
        }
        errorbuf = malloc (sizeof(char) * 1000);
        snprintf (errorbuf, 999, "[SQL/Statement Error @ %s]%s, SQLSTATE=%s\n", where, buf, sqlstate);

        fprintf (stderr, "[SQL/Statement Error @ %s]%s, SQLSTATE=%s\n", where, buf, sqlstate);
    }

    /*
     *  Get connection errors
     */
    while (SQLError (jobrep_henv, jobrep_hdbc, SQL_NULL_HSTMT, sqlstate, NULL, buf, sizeof(buf), NULL) == SQL_SUCCESS)
    {
        errorbuf = malloc (sizeof(char) * 1000);
        snprintf (errorbuf, 999, "[Connection Error @ %s]%s, SQLSTATE=%s\n", where, buf, sqlstate);
        
        fprintf (stderr, "[Connection Error @ %s]%s, SQLSTATE=%s\n", where, buf, sqlstate);
    }

    /*
     *  Get environmental errors
     */
    while (SQLError (jobrep_henv, SQL_NULL_HDBC, SQL_NULL_HSTMT, sqlstate, NULL, buf, sizeof(buf), NULL) == SQL_SUCCESS)
    {
        errorbuf = malloc (sizeof(char) * 1000);
        snprintf (errorbuf, 999, "[Environmental Error @ %s]%s, SQLSTATE=%s\n", where, buf, sqlstate);

        fprintf (stderr, "[Environmental Error @ %s]%s, SQLSTATE=%s\n", where, buf, sqlstate);
    }

    return errorbuf;
}


/******************************************************************************
Function:   SQL_Query
Description:
    Executes a SQL Statement via the ODBC connection
Parameters:
    char * request : the name of the plugin
Returns:
    0         : succes
    1 or else : failure
******************************************************************************/

int SQL_Query (char * request)
{
/*  Debug */

    int ParamCount = 0;
    int res = 0;

//    SQLUSMALLINT iPar = 0;



//    int i          = 0;
//    printf(": %s :: Now executing: \n", getenv("JOB_REPOSITORY_ID"));
//    for (i = 0; (request[i] != '\0'); i++)
//    {
//        if (((i % 150) == 0) && (i != 0))
//            printf("\n");
//
//        printf("%c", request[i]);
//    }
//    printf("\n\n");

   
    res = SQLAllocHandle(SQL_HANDLE_STMT, jobrep_hdbc, &jobrep_hstmt);
    if ((res != SQL_SUCCESS) && (res != SQL_SUCCESS_WITH_INFO))
    {
        ODBC_Errors("Failure in AllocStatement\n");
        return -1;

    }

    res = SQLExecDirect(jobrep_hstmt, request, SQL_NTS);
    if (res != SQL_SUCCESS) 
    {
        ODBC_Errors("SQLExecDirect");
        return -2;
    }
    else
    {
        return 0;
    }


    /*
     *  Close statement if openend
     */

    if (jobrep_hstmt)
    {
        int sts;
        sts = SQLCloseCursor (jobrep_hstmt);
        if (sts != SQL_ERROR)
            ODBC_Errors ("CloseCursor");
    }


    /*
     * Bug Fix on the API level: handling of ' must be a \' in the Query to be handled by the (MySQL) parser.
     * Filtering and morphing the string into a correct string with clear dilimitation.
     */
 
    // Code warning: need an open close parser ...


    /*
     *  Prepare & Execute the statement
     */

    if ((res = SQLPrepare (jobrep_hstmt, (UCHAR *) request, SQL_NTS)) != SQL_SUCCESS)
    {
        fprintf(stderr, "SQLPrepare : %d\n", res);
        ODBC_Errors ("SQLPrepare");
        return (-1);
    }


    /*
     *  Parameter binding
     */

    SQLNumParams (jobrep_hstmt, (SQLSMALLINT *) &ParamCount);

    if (ParamCount > 0)
    {

        if (globalParameters) 
        {
            SQLINTEGER returnValue = 0;

//            printf("Certificate: \n\n%s\n\n", globalParameters->value);

            res = SQLBindParameter (jobrep_hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 
                                           strlen (globalParameters->value), 0, globalParameters->value, 
                                           strlen (globalParameters->value) + 1, &returnValue);
            if (res != SQL_SUCCESS)
            {
                ODBC_Errors ("SQLBindParam");
                return (-1);
            }
//            printf("\n>>>\nreturnValue : %d\n>>>\n", returnValue);
        }
    }


    /*
     *  Execution of the statement
     */

    if ((res = SQLExecute (jobrep_hstmt)) != SQL_SUCCESS)
    {
        if (res != SQL_NEED_DATA)
        {
            ODBC_Errors ("SQLExec");
            return (-1);
        }
        fprintf(stderr, "SQLExecute: SQL_NEED_DATA\n");
    }


    // Clean up
//    if (globalParameters)
//    {
//        free(globalParameters->id);
//        free(globalParameters->value);
//        free(globalParameters);
//        globalParameters = NULL;
//    }


    return (0);
}


/******************************************************************************
Function:   SQL_SetParameter
Description:
    Let's you fill the value of a set parameter in a SQL Query.
    Use of these parameters could solve problems with SQL delimiters in the
    query string like the ' character.
Parameters:
    
Returns:
    0         : succes (no results)
  < 0 like -1 : failure
******************************************************************************/

int SQL_SetParameter (char * value)
{
    globalParameters = (TParamType *) malloc(sizeof(TParamType));
    globalParameters->id     = strdup("cert");
    globalParameters->value  = strdup(value);
    globalParameters->type   = SQL_VARCHAR;

    return 0;
}

/******************************************************************************
Function:   SQL_GetQueryResult
Description:
    Recieves the just executed query in a TResultSet structure
Parameters:
    TResultSet ** result
Returns:
    0         : succes (no results)
  < 0 like -1 : failure
  > 0         : success and the rowcount of the result in the return
Also:
    You'll get a fancy TResultSet structure dangling from a pointer.
******************************************************************************/

int SQL_GetQueryResult(TResultSet ** result)
{
    //static   TResultSet * resultSet = NULL;
 
    int      i;
    char     fetchBuffer[1000];
    short    colNum;
    int      rowNum = 0;
    char     colName[50];
    short    colType;
    UDWORD   colPrecision;
    SDWORD   colIndicator;
    short    colScale;
    short    colNullable;
    //UDWORD totalSets = 0;
    int sts = SQL_NO_DATA_FOUND;
 
    short tmpColCnt = 0;
    SQLINTEGER tmpRowCnt = 0;
 
    // Cleaning the pointer mocht er iets aanhangen.
    if (*result != NULL)
    {
//        result[0]->colCnt = 0;
//        result[0]->rowCnt = 0;
//        TResultSet_free(*result);
        *result = NULL;
    }

    /*
     *  Get the number of result columns for this cursor.
     *  If it is 0, then the statement was probably a select
     */
    if (SQLRowCount (jobrep_hstmt, &tmpRowCnt) != SQL_SUCCESS)
    {
        ODBC_Errors ("SQLRowCount");
        goto endCursor;
    }
 
    if (SQLNumResultCols (jobrep_hstmt, &tmpColCnt) != SQL_SUCCESS)
    {
        ODBC_Errors ("SQLNumResultCols");
        goto endCursor;
    }
 
    // What kinda result has the query produced?
    if (tmpColCnt == 0)
    {
        // Execution without any results
        //printf ("Statement executed.\n");
        goto endCursor;
    }
 
 
//    resultSet = (TResultSet *) realloc (resultSet, sizeof(TResultSet));
    *result = (TResultSet *) malloc (sizeof(TResultSet));
    (*result[0]).colCnt  = tmpColCnt;
    (*result[0]).rowCnt  = tmpRowCnt;
    (*result[0]).columns = (TColumn *) malloc (sizeof(TColumn) * (*result[0]).colCnt);

    (*result[0]).data = (TRow *) malloc (sizeof(TRow) * (*result[0]).rowCnt);
    for (i = 0; i < (*result[0]).rowCnt; i++)
    {
        (*result[0]).data[i] = (TField *) malloc(sizeof(TField) * (*result[0]).colCnt);
    }

//    resultSet = (TResultSet *) malloc (sizeof(TResultSet));
//    resultSet[0].colCnt  = tmpColCnt;
//    resultSet[0].rowCnt  = tmpRowCnt;
//    resultSet[0].columns = (TColumn *) malloc (sizeof(TColumn) * *resultSet[0].colCnt);
 
//    resultSet[0].data = (TRow *) malloc (sizeof(TRow) * resultSet[0].rowCnt);
//    for (i = 0; i < resultSet[0].rowCnt; i++)
//    {
//        resultSet[0].data[i] = (TField *) malloc(sizeof(TField) * resultSet[0].colCnt);
//    }
 
 
    // Let's get us a dataset
    while (1)
    {
        // Make a new row if there is data
        sts = SQLFetch (jobrep_hstmt);
        if (sts == SQL_NO_DATA_FOUND)
            break;
 
        if (sts != SQL_SUCCESS)
        {
            ODBC_Errors ("Fetch");
            break;
        }
 
        // Building the fields in the row(s)
//        for (colNum = 0; colNum < resultSet[0].colCnt; colNum++)
        for (colNum = 0; colNum < (*result[0]).colCnt; colNum++)
        {
            if (SQLDescribeCol (jobrep_hstmt, colNum + 1, (UCHAR *) colName, sizeof (colName),
                                NULL, &colType, &colPrecision,
                                &colScale, &colNullable) != SQL_SUCCESS)
            {
                ODBC_Errors ("SQLDescribeCol");
                goto endCursor;
            }
 
            // Fetch column data for this row
            if (SQLGetData (jobrep_hstmt, colNum + 1, SQL_CHAR, fetchBuffer, sizeof (fetchBuffer), &colIndicator) != SQL_SUCCESS)
            {
                ODBC_Errors ("SQLGetData");
                goto endCursor;
            }
 
            //resultSet[rowNum][colNum] = strdup(fetchBuffer);
//            resultSet[0].columns[colNum].columnname     = strdup(colName);
//            resultSet[0].columns[colNum].type           = colType;
//            resultSet[0].data[rowNum][colNum].fieldname = strdup(colName);
//            resultSet[0].data[rowNum][colNum].value     = strdup(fetchBuffer);
//            resultSet[0].data[rowNum][colNum].type      = colType;
 
            (*result[0]).columns[colNum].columnname     = strdup(colName);
            (*result[0]).columns[colNum].type           = colType;
            (*result[0]).data[rowNum][colNum].fieldname = strdup(colName);
            (*result[0]).data[rowNum][colNum].value     = strdup(fetchBuffer);
            (*result[0]).data[rowNum][colNum].type      = colType;
        }
        rowNum++;
    }
 
    endCursor:
        SQLFreeStmt (jobrep_hstmt, SQL_CLOSE);
 
//    *result = resultSet;
 
    return (*result[0]).rowCnt;
//    return resultSet[0].rowCnt;
}


/******************************************************************************
Function:   TResultSet_free
Description:
    Frees a TResultSet structure.
Parameters:
    TResultSet **
Returns:
    void
******************************************************************************/

void TResultSet_free (TResultSet * resultSet)
{
    int i = 0, j = 0;

    //printf(">>> TResultSet_free\n");
    return;


    if (resultSet == NULL)
        return;

    //printf(">>> TResultSet_free 2\n");

    if ((resultSet->colCnt <= 0) && (resultSet->rowCnt <= 0))
    {
        free(resultSet);
        resultSet = NULL;
        return;
    }
    //printf(">>> TResultSet_free 3\n");

    // Free columnnames
    for (i = 0; i < resultSet[0].colCnt; i++)
    {
        if ((resultSet[0].columns[i].columnname != NULL) && (strlen(resultSet[0].columns[i].columnname) > 1))
        {
//            free(resultSet[0].columns[i].columnname);
        }
    }
    //printf(">>> TResultSet_free 4\n");

    // Free data from all TField structures with it's TField structs
    for (i = 0; i < resultSet[0].rowCnt; i++)
    {
        for (j = 0; j < resultSet[0].colCnt; j++)
        {
            free(resultSet[0].data[i][j].fieldname);
            free(resultSet[0].data[i][j].value);
        }
        free(resultSet[0].data[i]);
    }
    
    // Free the rows where the TFields were and also the columnnames
    // As a bonus free the resultSet itself and make it NULL

    if (resultSet[0].data) free(resultSet[0].data);
    if (resultSet[0].columns) free(resultSet[0].columns);
    free(resultSet);

    resultSet = NULL;
}


/******************************************************************************
Function:   TSimpleStack_free
Description:
    Frees a TSimpleStack structure.
Parameters:
    TSimpleCert **, int size (count of elements of TSimpleCert
Returns:
    void
******************************************************************************/

void TSimpleCert_free (TSimpleCert * simpleStack, int size)
{
    int i = 0;

    //printf("Begin TSimpleCert_free\n");
    for (i = 0; i < size; i++)
    {
        if ((simpleStack[i]).X509cert) X509_free((simpleStack[i]).X509cert);       //printf("X509cert done\n");
        if ((simpleStack[i]).certStr)       free((simpleStack[i]).certStr);        //printf("certStr done\n");
//        if ((simpleStack[i]).subject_name)  free((simpleStack[i]).subject_name); fprintf("subj done\n");
//        if ((simpleStack[i]).issuer_name)   free((simpleStack[i]).issuer_name);  fprintf("issu done\n");
//        if ((simpleStack[i]).strNotBefore)  free((simpleStack[i]).strNotBefore); fprintf("before done\n");
//        if ((simpleStack[i]).strNotAfter)   free((simpleStack[i]).strNotAfter);  fprintf("after done\n");
    }
    //free(simpleStack);

    simpleStack = NULL;
//    fprintf("Eind TSimpleCert_free\n");
}


/******************************************************************************
Function:   SQL_Commit
Description:
    Commits the active transaction
Parameters:
Returns:
    SQL_SUCCESS : succes (no results)
   !SQL_SUCCESS : failure
******************************************************************************/

int SQL_Commit ()
{
    return SQLTransact(jobrep_henv, jobrep_hdbc, SQL_COMMIT);
}

/******************************************************************************
Function:   SQL_Rollback
Description:
    Rolls back the transaction
Parameters:
Returns:
    SQL_SUCCESS : succes (no results)
   !SQL_SUCCESS : failure
******************************************************************************/

int SQL_Rollback ()
{
    return SQLTransact(jobrep_henv, jobrep_hdbc, SQL_ROLLBACK);
}

/******************************************************************************
Function:   SQL_printfResultSet
Description:
    prints the resultSet to the stdout
Parameters:
    TResultSet * result
Returns:
    0        : succes (no results)
   !0        : failure
******************************************************************************/

int SQL_printfResultSet (TResultSet * resultSet)
{
     return SQL_fprintfResultSet (stdout, resultSet);
}
 
/******************************************************************************
Function:   SQL_fprintfResultSet
Description:
    prints the resultSet to the filestream, 'stream'
Parameters:
    TResultSet * result
    FILE *       stream
Returns:
    0        : succes (no results)
   !0        : failure
******************************************************************************/

int SQL_fprintfResultSet (FILE * stream, TResultSet * resultSet)
{
    int i, j;
 
    if (resultSet == NULL)
        return -1;
 
    if ( (resultSet[0].colCnt == 0) || ( resultSet[0].rowCnt == 0) )
        return EMPTYSET;
 
    for (j = 0; j < resultSet[0].colCnt; j++)
    {
        fprintf(stdout, "|%25s|", resultSet[0].columns[j].columnname);
    }
 
    fprintf(stdout, "\n");
    for (i = 0; i < (resultSet[0].colCnt * 25 + resultSet[0].colCnt + 3); i++)
        fprintf(stdout, "-");
    fprintf(stdout, "\n");
 
 
    for (i = 0; i < resultSet[0].rowCnt; i++)
    {
        for (j = 0; j < resultSet[0].colCnt; j++)
        {
            fprintf(stdout, "|%25s|", resultSet[0].data[i][j].value);
        }
        fprintf(stdout, "\n");
    }
 
    return 0;
}



/******************************************************************************
Function:   SQL_GetValue
Description:
    returns a value that you want from a resultset so that you don't have
    to do this yourself.
    Note: for know it will only return you a first occurance of your wanted 
          value
    
Parameters:
    TResultSet * result
    char       * columnName
Returns:
    char ** value : first occurance of value in specified column
    0             : succes (no results)
   !0             : failure
******************************************************************************/

int SQL_GetValue (TResultSet * resultSet, char * columnName, char ** value)
{
    int i = 0;

    free(*value);
    *value = NULL;
    for (i = 0; i < resultSet[0].colCnt; i++)
    {
        if (strncmp(resultSet[0].columns[i].columnname, columnName, strlen(resultSet[0].columns[i].columnname)) == 0)
        {
            *value = strdup(resultSet[0].data[0][i].value);
            return 0;
        }
    }
    return -1;
}


/******************************************************************************
Function:   SQL_SetLock
Description:
    Sets a lock on the given table with the kindOfLock Specified.
 
Parameters:
    char       * table
    int          kindOfLock (LOCK_READ, LOCK_WRITE)
Returns:
    0             : succes (no results)
   !0             : failure
******************************************************************************/

int SQL_SetLock (char * table, int kindOfLock)
{
    char R[1000];
    char W[1000];

    if (kindOfLock == LOCK_WRITE)
    {
        sprintf(W, "lock tables %s write", table);
        return SQL_Query(W);
    }

    else if (kindOfLock == LOCK_READ)
    {
        sprintf(R, "lock tables %s read", table);
        return SQL_Query(R);
    }

    else if (kindOfLock == (LOCK_READ + LOCK_WRITE))
    {
        sprintf(R, "lock tables %s read", table); 
        sprintf(W, "lock tables %s write", table);

        return (SQL_Query(R) + SQL_Query(W));
    }

    return -1;
}


/******************************************************************************
Function:   SQL_UnLock
Description:
    Unlocks the tables that are lock by this connection.

Parameters:
Returns:
    0             : succes (no results)
   !0             : failure
******************************************************************************/

int SQL_UnLock()
{
    return  SQL_Query("unlock tables");
}
